==> herdstat-1.1.1_p7.tar.bz2: herdstat-1.1.1_p7/src/action_which_handler.cc <==
/*
 * herdstat -- src/action_which_handler.cc
 * $Id: action_which_handler.cc 502 2005-08-30 13:57:46Z ka0ttic $
 * Copyright (c) 2005 Aaron Walker <ka0ttic@gentoo.org>
 *
 * This file is part of herdstat.
 *
 * herdstat is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * herdstat is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * herdstat; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 325, Boston, MA  02111-1257  USA
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <algorithm>

#include "pkgcache.hh"
#include "action_which_handler.hh"

int
action_which_handler_T::operator() (opts_type &opts)
{
    pkgcache_T pkgcache;
    std::multimap<std::string, std::string>::iterator m;

    if (all)
    {
        std::cerr << "which action handler does not support the 'all' target."
            << std::endl;
        return EXIT_FAILURE;
    }
    else if (regex and opts.size() > 1)
    {
        std::cerr << "You may only specify one regular expression."
            << std::endl;
        return EXIT_FAILURE;
    }
    else if (regex)
    {
        util::regex_T::string_type re(opts.front());
        opts.clear();

        pkgcache.init(portdir);
        regexp.assign(re, eregex? REG_EXTENDED|REG_ICASE : REG_ICASE);
        matches = portage::find_package_regex(config, regexp,
                    overlay, &search_timer, pkgcache);

        if (matches.empty())
        {
            std::cerr << "Failed to find any packages matching '" << re << "'."
                << std::endl;
            return EXIT_FAILURE;
        }

        for (m = matches.begin() ; m != matches.end() ; ++m)
        {
            if (std::find(opts.begin(), opts.end(), m->second) == opts.end())
                opts.push_back(m->second);
            else
            {
                matches.erase(m);
                --m;
            }
        }
    }
    else
    {
        opts_type::iterator i;
        for (i = opts.begin() ; i != opts.end() ; ++i)
            matches.insert(std::make_pair("", *i));
    }

    for (m = matches.begin() ; m != matches.end() ; ++m)
    {
        std::string ebuild;
        std::pair<std::string, std::string> p;

        try
        {
            if (regex)
                p = *m;
            else
                p = portage::find_package(config, m->second,
                    overlay, &search_timer);
        }
        catch (const portage::ambiguous_pkg_E &e)
        {
            std::cerr << e.name()
                << " is ambiguous. Possible matches are: "
                << std::endl << std::endl;
            
            std::vector<std::string>::const_iterator i;
            for (i = e.packages.begin() ; i != e.packages.end() ; ++i)
            {
                if (optget("quiet", bool) or not optget("color", bool))
                    std::cerr << *i << std::endl;
                else
                    std::cerr << color[green] << *i << color[none] << std::endl;
            }

            if (matches.size() == 1)
                return EXIT_FAILURE;

            continue;
        }
        catch (const portage::nonexistent_pkg_E &e)
        {
            std::cerr << m->second << " doesn't seem to exist." << std::endl;

            if (matches.size() == 1)
                return EXIT_FAILURE;

            continue;
        }

//        try
//        {
//            /* try p.first (may be an overlay) first */
//            ebuild = portage::ebuild_which(p.first, p.second);
//        }
//        catch (const portage::nonexistent_pkg_E)
//        {
//            /* nope, so use real PORTDIR */
//            ebuild = portage::ebuild_which(portdir, p.second);
//        }

        if (regex)
            ebuild = portage::ebuild_which(config, p.second, overlay, NULL,
                pkgcache);
        else
            ebuild = portage::ebuild_which(config, p.second, overlay, NULL);

        if (not count)
            *stream << ebuild << std::endl;
    }

    size = matches.size();

    flush();
    return EXIT_SUCCESS;
}

/* vim: set tw=80 sw=4 et : */

==> herdstat-1.1.2.tar.bz2: herdstat-1.1.2/src/action_which_handler.cc <==
/*
 * herdstat -- src/action_which_handler.cc
 * $Id: action_which_handler.cc 801 2005-11-02 15:00:00Z ka0ttic $
 * Copyright (c) 2005 Aaron Walker <ka0ttic@gentoo.org>
 *
 * This file is part of herdstat.
 *
 * herdstat is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * herdstat is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * herdstat; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 325, Boston, MA  02111-1257  USA
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <algorithm>
#include <herdstat/portage/exceptions.hh>
#include <herdstat/portage/find.hh>
#include "pkgcache.hh"
#include "action_which_handler.hh"

using namespace herdstat;
using namespace herdstat::util;

action_which_handler::~action_which_handler()
{
}

int
action_which_handler::operator() (opts_type &opts)
{
    if (not options::fields().empty())
    {
        std::cerr << "--field doesn't really make much sense with --away."
            << std::endl;
        return EXIT_FAILURE;
    }

    if (options::all())
    {
        std::cerr << "which action handler does not support the 'all' target."
            << std::endl;
        return EXIT_FAILURE;
    }

    pkgcache pkgcache;
    std::multimap<std::string, std::string>::iterator m;

    if (options::regex())
    {
        regexp.assign(opts.front());
        opts.clear();

        pkgcache.init();
        matches = portage::find_package_regex(regexp, options::overlay(),
                    &search_timer, pkgcache);

        if (matches.empty())
        {
            std::cerr << "Failed to find any packages matching '"
                << regexp() << "'." << std::endl;
            return EXIT_FAILURE;
        }

        for (m = matches.begin() ; m != matches.end() ; ++m)
        {
            if (std::find(opts.begin(), opts.end(), m->second) == opts.end())
                opts.push_back(m->second);
            else
                matches.erase(m--);
        }
    }
    else
    {
        for (opts_type::iterator i = opts.begin() ; i != opts.end() ; ++i)
            matches.insert(std::make_pair("", *i));
    }

    for (m = matches.begin() ; m != matches.end() ; ++m)
    {
        std::string ebuild;
        std::pair<std::string, std::string> p;

        try
        {
            if (options::regex())
                p = *m;
            else
                p = portage::find_package(m->second, options::overlay(),
                        &search_timer);
        }
        catch (const portage::AmbiguousPkg &e)
        {
            std::cerr << e.name()
                << " is ambiguous. Possible matches are: "
                << std::endl << std::endl;
            
            std::vector<std::string>::const_iterator i;
            for (i = e.packages.begin() ; i != e.packages.end() ; ++i)
            {
                if (options::quiet() or not options::color())
                    std::cerr << *i << std::endl;
                else
                    std::cerr << color[green] << *i << color[none] << std::endl;
            }

            if (matches.size() == 1)
                return EXIT_FAILURE;

            continue;
        }
        catch (const portage::NonExistentPkg &e)
        {
            std::cerr << m->second << " doesn't seem to exist." << std::endl;

            if (matches.size() == 1)
                return EXIT_FAILURE;

            continue;
        }

        if (options::regex())
            ebuild = portage::ebuild_which(p.second, options::overlay(), NULL,
                pkgcache);
        else
            ebuild = portage::ebuild_which(p.second, options::overlay(), NULL);

        if (not options::count())
            *stream << ebuild << std::endl;
    }

    size = matches.size();

    flush();
    return EXIT_SUCCESS;
}

/* vim: set tw=80 sw=4 et : */
